# SPDX-License-Identifier: GPL-2.0
# Copyright (C) 2025 Bardia Moshiri <bardia@furilabs.com>

import subprocess
from pathlib import Path
import threading

_command_lock = threading.Lock()
HELPER_PATH = "/usr/libexec/bootman-helper"

def run_helper(*args):
    """
    Run the bootman helper script with the given arguments.

    Args:
        *args: Arguments to pass to the helper script

    Returns:
        tuple: (success_boolean, message)
    """
    global _command_lock

    with _command_lock:  # Ensure only one privileged operation runs at a time
        try:
            result = subprocess.run(
                ['pkexec', HELPER_PATH, *map(str, args)],
                capture_output=True,
                text=True
            )

            if result.returncode != 0:
                return False, f"Command execution failed: {result.stderr}"
            return True, result.stdout
        except Exception as e:
            return False, f"Error executing command: {str(e)}"

def mount_partition():
    """Mount the FuriOS persist partition."""
    return run_helper("mount")

def get_partition_size(partition_name):
    """Get the size of a specific LVM partition."""
    success, output = run_helper("lvdisplay", f"/dev/droidian/{partition_name}")

    if success:
        for line in output.splitlines():
            if "LV Size" in line:
                size = line.split()[2]
                unit = line.split()[3]
                return f"{size}{unit}"
    return "Unknown"

def write_partitions_file(partitions):
    """Write partitions to /furios_persist/bootman/partitions file."""
    content = ''
    for partition in partitions:
        display_name = process_partition_name(partition)
        content += f"{partition}:{display_name}\n"

    return run_helper("write_partitions", content)

def create_install_commands(name, size):
    """Create commands for creating a new partition."""
    partition_name = name.replace(" ", "-").lower()

    # Get current root filesystem size
    success, output = run_helper("lvdisplay", "/dev/droidian/droidian-rootfs")
    if not success:
        return False, "Failed to get current partition size"

    current_size = None
    for line in output.splitlines():
        if "LV Size" in line:
            current_size = float(line.split()[2])
            break

    if current_size is None:
        return False, "Could not determine current partition size"

    # Calculate new sizes
    new_size = current_size - float(size)
    new_size_mb = int(new_size * 1024)
    size_mb = int(float(size) * 1024)

    # Write the command file
    commands = [
        "e2fsck -p -y -f /dev/droidian/droidian-rootfs",
        f"resize2fs /dev/droidian/droidian-rootfs {new_size_mb}M",
        f"lvm lvreduce -L -{size_mb}M -r /dev/droidian/droidian-rootfs",
        f"lvm lvcreate -L {size_mb}M -n {partition_name} droidian"
    ]
    success, _ = run_helper("write_commands", "\n".join(commands))
    if not success:
        return False, "Failed to write commands"

    # Write the wip file
    success, _ = run_helper("write_wip", f"{partition_name}:{name}")
    if not success:
        return False, "Failed to write wip file"

    return True, "Partition creation queued successfully"

def delete_install_commands(partition_name):
    """Create commands for deleting a partition."""
    # Get partition size
    success, output = run_helper("lvdisplay", f"/dev/droidian/{partition_name}")
    if not success:
        return False, "Failed to get partition size"

    size = None
    for line in output.splitlines():
        if "LV Size" in line:
            size = float(line.split()[2])
            unit = line.split()[3]
            if unit.lower() == 'gib':
                size = size * 1024
            break

    if size is None:
        return False, "Could not determine partition size"

    # Write the command file
    commands = [
        f"lvm lvremove -f /dev/droidian/{partition_name}",
        f"lvm lvextend -L +{int(size)}M /dev/droidian/droidian-rootfs",
        "resize2fs /dev/droidian/droidian-rootfs"
    ]
    success, _ = run_helper("write_commands", "\n".join(commands))
    if not success:
        return False, "Failed to write commands"

    # Update partitions file
    success, output = run_helper("cat", "/furios_persist/bootman/partitions")
    if success:
        partitions = [line.strip() for line in output.splitlines() if partition_name not in line]
        content = "\n".join(partitions)
        success, _ = run_helper("write_partitions", content)
        if not success:
            return False, "Failed to update partitions file"

    return True, "Deletion queued successfully"

def is_mounted(mount_point):
    """Check if a mount point is currently mounted."""
    try:
        return Path('/proc/mounts').read_text().find(mount_point) != -1
    except Exception:
        return False

def is_partition_mounted(partition_name):
    """Check if a specific partition is currently mounted."""
    try:
        return Path('/proc/mounts').read_text().find(f"/dev/droidian/{partition_name}") != -1
    except Exception:
        return False

def process_partition_name(partition_name):
    """Convert a partition name to a more readable format."""
    name = partition_name.replace('droidian-', '').replace('furios-', '')
    words = name.split('-')
    return ' '.join(word.capitalize() for word in words)

def list_partitions():
    """List all available partitions."""
    try:
        droidian_path = Path("/dev/droidian")
        if not droidian_path.exists():
            return []

        excluded = ['droidian-persistent', 'droidian-reserved']
        return [p.name for p in droidian_path.iterdir() 
                if p.exists() and p.name not in excluded]
    except Exception:
        return []

def read_partitions_file(partitions_file):
    """Read and parse the partitions file."""
    try:
        content = Path(partitions_file).read_text().strip()
        return [line.strip().split(':')
                for line in content.split('\n')
                if ':' in line]
    except Exception:
        return []

def get_queued_partition():
    """Check for any queued partition installations."""
    try:
        wip_file = Path("/furios_persist/bootman/wip-partitions")
        commands_file = Path("/furios_persist/bootman/commands")

        if not (wip_file.exists() and commands_file.exists()):
            return None

        content = wip_file.read_text().strip()
        if not content:
            return None

        for line in content.split('\n'):
            if ':' in line:
                return tuple(line.strip().split(':'))

        return None
    except Exception:
        return None
